/**
 ***********************************************************************
 * $Id: Control.cs 330 2015-01-06 08:08:07Z mkwayisi $
 * ---------------------------------------------------------------------
 * Authored by Michael Kwayisi. Copyright (c) 2014. See license below.
 * Comments are appreciated - mic at kwayisi dot org | www.kwayisi.org
 * ---------------------------------------------------------------------
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are stringently met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions, and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions, and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. The end-user documentation included with the redistribution,
 *     if any, must include the following acknowledgment:
 *       "This product includes software written by Michael Kwayisi."
 *     Alternately, this acknowledgment may appear in the software
 *     itself, if and wherever such 3rd-party acknowledgments appear.
 *  4. Neither the name of the software nor the name of its author
 *     and/or contributors may be used to endorse or promote products
 *     derived from this software without specific prior permission.
 ***********************************************************************
 */

using System;
using System.Drawing;

public class Control
{
//======================================================================
// [DATA] Class instance data fields.
public Size         Size        = new Size(0, 0);
public Point        Location    = new Point(0, 0);
public Point        Padding     = new Point(0, 0);
public ConsoleColor ForeColor   = ConsoleColor.Gray;
public ConsoleColor BorderColor = ConsoleColor.Gray;
public BorderStyle  BorderStyle = BorderStyle.None;
protected string[] text = new string[] {};

//======================================================================
// [PROP] Bounds - Gets the bounding rectangle of the control.
public Rectangle Bounds {
	get {
		return new Rectangle(this.Location, this.Size);
	}
}

//======================================================================
// [PROP] ClientSize - Gets the client size of the control.
public Size ClientSize {
	get {
		return new Size(this.Size.Width - (this.BorderStyle == BorderStyle.None ? 0 : 2),
			this.Size.Height - (this.BorderStyle == BorderStyle.None ? 0 : 2));
	}
}

//======================================================================
// [PROP] Text - Gets or sets the text for this control.
public string Text {
	get {
		return (this.text.Length > 0 ? this.text[0] : "");
	}
	set {
		this.TextEx = new string[] {value};
	}
}

//======================================================================
// [PROP] TextEx - Gets or sets an extended text (array) for this control.
public string[] TextEx {
	get {
		return this.text;
	}
	set {
		this.text = value;
	}
}

//======================================================================
// [FUNC] A protected method that draws the border of a control on-screen.
protected void drawBorder()
{
	if (this.BorderStyle != BorderStyle.None) {
		Console.ForegroundColor = this.BorderColor;
		if (this.Size.Height == 1 && this.Size.Width >= 2) {
			Console.SetCursorPosition(this.Location.X, this.Location.Y);
			Console.Write(
				(this.BorderStyle == BorderStyle.Single ||
					this.BorderStyle == BorderStyle.SingleCorners) ?
				Lib.CharSLC : Lib.CharDLC);
			Console.Write("".PadRight(this.Size.Width - 2,
				(this.BorderStyle == BorderStyle.Single ||
					this.BorderStyle == BorderStyle.DoubleCorners) ?
				Lib.CharSHZ : Lib.CharDHZ));
			Console.Write(
				(this.BorderStyle == BorderStyle.Single ||
					this.BorderStyle == BorderStyle.SingleCorners) ?
				Lib.CharSRC : Lib.CharDRC);
		} else if (this.Size.Width >= 2) {
			for (int i = 0; i < this.Size.Height; i++) {
				Console.SetCursorPosition(this.Location.X, this.Location.Y + i);
				if (i == 0) {
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.SingleCorners) ?
						Lib.CharSLT : Lib.CharDLT);
					Console.Write("".PadRight(this.Size.Width - 2,
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.DoubleCorners) ?
						Lib.CharSHZ : Lib.CharDHZ));
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.SingleCorners) ?
						Lib.CharSRT : Lib.CharDRT);
				} else if (i < this.Size.Height - 1) {
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.DoubleCorners) ?
						Lib.CharSVT : Lib.CharDVT);
					Console.CursorLeft += this.Size.Width - 2;
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.DoubleCorners) ?
						Lib.CharSVT : Lib.CharDVT);
				} else {
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.SingleCorners) ?
						Lib.CharSLB : Lib.CharDLB);
					Console.Write("".PadRight(this.Size.Width - 2,
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.DoubleCorners) ?
						Lib.CharSHZ : Lib.CharDHZ));
					Console.Write(
						(this.BorderStyle == BorderStyle.Single ||
							this.BorderStyle == BorderStyle.SingleCorners) ?
						Lib.CharSRB : Lib.CharDRB);
				}
			}
		}
	}
}

//======================================================================
// [FUNC] A private method that draws or clears the text of a "Control"
private void drawText(bool isClearText)
{
	int posX = this.Location.X + this.Padding.X +
		(this.BorderStyle == BorderStyle.None ? 0 : 1);
	int posY = this.Location.Y + this.Padding.Y +
		(this.BorderStyle == BorderStyle.None ? 0 : 1);
	Console.ForegroundColor = this.ForeColor;
	
	for (int i = 0; i < this.text.Length; i++) {
		Console.SetCursorPosition(posX, posY + i);
		Console.Write(isClearText ? "".PadRight(this.text[i].Length, ' ') : this.text[i]);
	}
}

//======================================================================
// [FUNC] Draws the text of a "Control".
public void DrawText()
{
	this.drawText(false);
}

//======================================================================
// [FUNC] A public channel to clear the text of a "Control".
public void ClearText()
{
	this.drawText(true);
}

//======================================================================
// [FUNC] Helps to center the text of a "Control". It's just shit!
public void CenterText()
{
	this.Padding.X = (this.ClientSize.Width - this.Text.Length) / 2;
	this.Padding.Y = (this.ClientSize.Height - this.TextEx.Length) / 2;
}

//======================================================================
// [FUNC] Displays a "Control" on the console screen.
public void Show()
{
	this.drawBorder();
	this.drawText(false);
}

//======================================================================
// [FUNC] Hides a "Control" from the console screen.
public void Hide()
{
	for (int i = 0; i < this.Size.Height; i++) {
		Console.SetCursorPosition(this.Location.X, this.Location.Y + i);
		Console.Write("".PadRight(this.Size.Width), ' ');
	}
}
}
