/**
 ***********************************************************************
 * $Id: Program.cs 330 2015-01-06 08:08:07Z mkwayisi $
 * ---------------------------------------------------------------------
 * Authored by Michael Kwayisi. Copyright (c) 2014. See license below.
 * Comments are appreciated - mic at kwayisi dot org | www.kwayisi.org
 * ---------------------------------------------------------------------
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are stringently met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions, and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions, and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. The end-user documentation included with the redistribution,
 *     if any, must include the following acknowledgment:
 *       "This product includes software written by Michael Kwayisi."
 *     Alternately, this acknowledgment may appear in the software
 *     itself, if and wherever such 3rd-party acknowledgments appear.
 *  4. Neither the name of the software nor the name of its author
 *     and/or contributors may be used to endorse or promote products
 *     derived from this software without specific prior permission.
 ***********************************************************************
 */

using System;
using System.IO;
using System.Drawing;
using Microsoft.Win32;
using System.Threading;
using System.Reflection;
using System.Diagnostics;
using System.Runtime.Serialization.Formatters.Binary;

//======================================================================
// [INFO] Some assembly information. I need to sign my work ;)
[assembly: AssemblyTitle(Program.Title)]
[assembly: AssemblyDescription(Program.Title)]
[assembly: AssemblyCompany(Program.Company)]
[assembly: AssemblyProduct(Program.Title)]
[assembly: AssemblyCopyright(Program.Copyright)]
[assembly: AssemblyVersion(Program.Version)]

public static class Program
{
//======================================================================
// [DATA] Static class's data fields.
public const string Title = "Mon Jeu De Console";
public const string Version = "1.0";
public const string Company = "MetaGhana Inc.";
public const string Copyright =
	"Copyright (c) 2012 Michael Kwayisi. All Rights Reserved.";

public  const  char     TokenMagicChar = '#';
public  const  int      MaxPlayerNameLength = 19;
private static bool     ImStillAlive = true;
private static bool     IsGameInProgress = false;
private static string   PlayerName = "";
private static int      PlayerMoves = 0;
private static string[] HighScores = new string[10];
private static string   HighScoresFilePath = "";
private static string   ExecAssemblyPath = "";

private static Control xHead = new Control();
private static Control xStat = new Control();
private static Control xMain = new Control();
private static Control xHelp = new Control();
private static Control xFoot = new Control();
private static Token[] Token = new Token[9];
private static ConsoleKeyInfo ReadKeyInfo;

//======================================================================
// [FUNC] Obviously, the assembly's entry point of execution.
private static void Main()
{
	if (!IsRegSetupOK() && SetupReg()) {
		Process.Start(ExecAssemblyPath);
		return;
	}
	
	Console.SetWindowSize(80, 31);
	Console.SetBufferSize(Console.WindowWidth, Console.WindowHeight);
	Console.CursorVisible = !true;
	Console.Title = Title + " - Written by Michael Kwayisi";
	
	ShowSplashScreen();
	LoadHighScores();
	PrepareScreen();

	do switch ((ReadKeyInfo = Console.ReadKey(true)).Key) {
		case ConsoleKey.Escape:
			QuitGame();
			break;
		case ConsoleKey.F1:
			if (IsGameInProgress)
				ShowHelp();
			break;
		case ConsoleKey.F2:
			ShowHighScores();
			break;
		case ConsoleKey.F5:
			StartGame();
			break;
		case ConsoleKey.LeftArrow:
		case ConsoleKey.UpArrow:
		case ConsoleKey.RightArrow:
		case ConsoleKey.DownArrow:
			if (IsGameInProgress)
				MoveToken(ReadKeyInfo.Key);
			break;
		case ConsoleKey.E:
			if (IsGameInProgress)
				CheckGame();
			break;
		default:
			// What's that key for?
			// Put some ketchup on it and have it for lunch!
			break;
	} while (ImStillAlive);
	
	// APP: But how did I got here? I thought I'll always be alive.
	// CPU: A member in this class touched one of your private parts.
	// APP: Really? Tell me, who did that?
	// CPU: Are you asking me? I should be asking you that instead.
	// APP: Oh no! Gimme one more chance! I'm still ALIVE!!!
	// CPU: Hehehe, it's too late. Always be careful of your class members!
	ImStillAlive = true;
}

//======================================================================
// [FUNC] Verifies if the registry settings for the executing assembly
// is OK. I just want that notorious "Raster Font" at 10 x 18 pixels.
private static bool IsRegSetupOK()
{
	RegistryKey regKey = null;
	ExecAssemblyPath = Assembly.GetExecutingAssembly().Location;
	try {
		regKey = Registry.CurrentUser.OpenSubKey("Console\\" +
			ExecAssemblyPath.Replace("\\", "_"));
		if (regKey == null || regKey.ValueCount != 4)
			return false;
		foreach (string valueName in regKey.GetValueNames()) {
			switch (valueName) {
				case "FontFamily":
					if ((int) regKey.GetValue(valueName, 0) != 0x30)
						return false;
					break;
				case "FontSize":
					if ((int) regKey.GetValue(valueName, 0) != 0x12000a)
						return false;
					break;
				case "ScreenBufferSize":
				case "WindowSize":
					if ((int) regKey.GetValue(valueName, 0) != 0x1f0050)
						return false;
					break;
				default:
					return false;
			}
		}
		return true;
	} catch (Exception) {
		return false;
	}
}

//======================================================================
// [FUNC] Sets the kinda registry settings I want for this assembly. Is
// this even necessary? I'm supposed to write a simple console
// application but here I am messing with someone's registry stuff :|
private static bool SetupReg()
{
	RegistryKey regKey = null;
	string subKey = "Console\\" + ExecAssemblyPath.Replace("\\", "_");
	try {
		try {
			Registry.CurrentUser.DeleteSubKeyTree(subKey);
		} catch (Exception) {}
		regKey = Registry.CurrentUser.CreateSubKey(subKey);
		regKey.SetValue("FontFamily", 0x30, RegistryValueKind.DWord);
		regKey.SetValue("FontSize", 0x12000a, RegistryValueKind.DWord);
		regKey.SetValue("ScreenBufferSize", 0x1f0050, RegistryValueKind.DWord);
		regKey.SetValue("WindowSize", 0x1f0050, RegistryValueKind.DWord);
		return true;
	} catch (Exception) {
		return false;
	}
}

//======================================================================
// [FUNC] Displays some stupid splash message on-screen before the start
// of the game. This is a last-minute filler so expected some dirty
// (dangerous?) code here. If Anders Hejlsberg should ever read this,
// guess what he'll say ;) BA-BA-BLACK SHEEP!
private static void ShowSplashScreen()
{
	Control ctrl = new Control();
	ctrl.BorderStyle = BorderStyle.SingleCorners;
	ctrl.BorderColor = ctrl.ForeColor = ConsoleColor.Cyan;
	ctrl.Size = new Size(Console.BufferWidth - 4, Console.BufferHeight - 6);
	ctrl.Location = new Point(2, 1);
	ctrl.Show();
	
	ctrl.Location.Y = ctrl.Size.Height + 2;
	ctrl.Size.Height = 3;
	ctrl.Text = "L O A D I N G . . .";
	ctrl.CenterText();
	ctrl.Show();
	
	Thread.Sleep(750);
	
	string[] tmp = Lib.GetStringMap("M O N   J E U   D E   C O N S O L E");
	string[] str = new string[5];
	str[0] = tmp[0];
	str[1] = tmp[1];
	str[2] = tmp[2];
	str[3] = "".PadRight(str[0].Length);
	str[4] = "<< ATTITUDE IS NO SUBSTITUTE FOR COMPETENCE - ESR >>";
	str[4] += "".PadRight((str[0].Length - str[4].Length) / 2);
	str[4] = "".PadRight(str[0].Length - str[4].Length) + str[4];
	int posX = (Console.BufferWidth - str[0].Length) / 2;
	int posY = (Console.BufferHeight - 3 - str.Length) / 2;
	bool isDone = true;
	Random rnd = new Random();
	Control[] bits = new Control[str.Length * str[0].Length];
	Console.ForegroundColor = ctrl.ForeColor;
	
	for (int i = 0, m = 0; i < str.Length; i++) {
		for (int j = 0; j < str[i].Length; j++, m++) {
			// "bits" are not really "Control"s but I just needed the
			// Padding (target location) and Location (current location)
			// properties ready instead of building a whole class for this
			// little kitty.
			bits[m] = new Control();
			bits[m].Padding = new Point(posX + j, posY + i);
			bits[m].Location = new Point(rnd.Next(posX, posX + str[i].Length),
				rnd.Next(posY, posY + str.Length));
			bits[m].Text = str[i][j].ToString();
		}
	}
	
	do {
		isDone = true;
		for (int i = 0; i < bits.Length; i++) {
			if (bits[i].Location.X != bits[i].Padding.X) {
				bits[i].Location.X += (bits[i].Location.X < bits[i].Padding.X ? 1 : -1);
				isDone = false;
			} else if (bits[i].Location.Y != bits[i].Padding.Y) {
				bits[i].Location.Y += (bits[i].Location.Y < bits[i].Padding.Y ? 1 : -1);
				isDone = false;
			}
			Console.SetCursorPosition(bits[i].Location.X, bits[i].Location.Y);
			Console.Write(bits[i].Text);
		}
	} while (!isDone);
	
	ctrl.Text = "P R E S S   [ E N T E R ]   T O   C O N T I N U E";
	ctrl.CenterText();
	ctrl.Show();
	
	while (Console.ReadKey(true).Key != ConsoleKey.Enter);
		Console.Clear();
}

//======================================================================
// [FUNC] Loads the game's high scores from a file.
private static void LoadHighScores()
{
	BinaryFormatter bf;
	FileStream fs = null;
	
	HighScoresFilePath =
		Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) +
			"\\" + Title + "\\HighScores.bin";
	
	try {
		bf = new BinaryFormatter();
		fs = new FileStream(HighScoresFilePath, FileMode.Open);
		HighScores = (string[]) bf.Deserialize(fs);	
	} catch (Exception) {
		for (int i = 0; i < HighScores.Length; i += 2) {
			HighScores[i] = "(EMPTY)";
			HighScores[i+1] = "NULL";
		}
	} finally {
		if (fs != null)
			fs.Close();
	}
}

//======================================================================
// [FUNC] Save the game's high scores into a file.
private static void SaveHighScores()
{
	BinaryFormatter bf;
	FileStream fs = null;
	string dir = "";
	try {
		dir = new FileInfo(HighScoresFilePath).DirectoryName;
		if (!Directory.Exists(dir))
			Directory.CreateDirectory(dir);
		bf = new BinaryFormatter();
		fs = new FileStream(HighScoresFilePath, FileMode.Create);
		bf.Serialize(fs, HighScores);
	} catch (Exception) {
		// "You say it best, when you say nothing at all." - Alison Krauss
	} finally {
		if (fs != null)
			fs.Close();
	}
}

//======================================================================
// [FUNC] Draws the various "Control"s on-screen and many other boring stuff.
private static void PrepareScreen()
{
	xHead.TextEx = Lib.GetStringMap("M O N    J E U    D E    C O N S O L E");
	xHead.BorderStyle = BorderStyle.SingleCorners;
	xHead.BorderColor = xHead.ForeColor = ConsoleColor.Cyan;
	xHead.Size = new Size(78, 7);
	xHead.Location.X = 1;
	xHead.CenterText();
	xHead.Show();
	
	for (int i = 0; i < Token.Length; i++) {
		Token[i] = new Token();
		Token[i].Value = i + 1;
		Token[i].CellID = i + 1;
		Token[i].Location.X = i % 3 == 0 ?
			xHead.Location.X : Token[i-1].Bounds.Right + 1;
		Token[i].Location.Y = i / 3 == 0 ?
			xHead.Bounds.Bottom : Token[i-3].Bounds.Bottom;
		Token[i].BorderColor = xHead.BorderColor;
		Token[i].ForeColor = ConsoleColor.Green;
		Token[i].DrawBorder();
		for (int j = 0; j < 5; j++) {
			Console.SetCursorPosition(Token[i].Location.X + 2,
				Token[i].Location.Y + j + 1);
			Console.Write("".PadRight(6, TokenMagicChar));
		}
	}
	
	xStat.BorderStyle = BorderStyle.DoubleCorners;
	xStat.BorderColor = xStat.ForeColor = xHead.BorderColor;
	xStat.Location = new Point(Token[2].Bounds.Right + 1, Token[2].Location.Y);
	xStat.Size = new Size(xHead.Bounds.Right - xStat.Location.X, 3);
	xStat.Padding.X++;
	
	xMain.Text = "WELCOME. PLEASE ENTER YOUR NAME:";
	xMain.BorderStyle = xStat.BorderStyle;
	xMain.BorderColor = xMain.ForeColor = xHead.BorderColor;
	xMain.Location = xStat.Location;
	xMain.Size = new Size(xStat.Size.Width,
		Token[8].Bounds.Bottom - xMain.Location.Y);
	xMain.CenterText();
	xMain.Padding.Y = 6;
	xMain.Show();
	
	xHelp.BorderStyle = xStat.BorderStyle;
	xHelp.BorderColor = xHelp.ForeColor = xStat.BorderColor;
	xHelp.Size = xStat.Size;
	xHelp.Location = new Point(xStat.Location.X, xMain.Bounds.Bottom - 3);
	xHelp.Padding = xStat.Padding;
	
	xFoot.Text = Copyright.ToUpper();
	xFoot.BorderStyle = xHead.BorderStyle;
	xFoot.BorderColor = xFoot.ForeColor = xHead.BorderColor;
	xFoot.Size = new Size(xHead.Size.Width, 3);
	xFoot.Location = new Point(xHead.Location.X,
		Token[Token.Length-1].Bounds.Bottom);
	xFoot.CenterText();
	xFoot.Show();

	do {
		Control ctrl = new Control();
		ctrl.BorderStyle = BorderStyle.Single;
		ctrl.BorderColor = ctrl.ForeColor = xMain.BorderColor;
		ctrl.Size = new Size(xMain.Size.Width - 6, 3);
		ctrl.Location = new Point(xMain.Location.X + 3, xMain.Location.Y + 10);
		ctrl.Padding.X = 1;
		ctrl.Show();
		
		Console.SetCursorPosition(ctrl.Location.X + ctrl.Padding.X + 1,
			ctrl.Location.Y + 1);
	} while (false);
	
	PlayerName = Lib.ReadLine(MaxPlayerNameLength);
	xStat.Text = "PLAYER: " +
		PlayerName.PadRight(MaxPlayerNameLength, ' ') + "   MOVES: 0000";
	xMain.Hide();
	xMain.Location.Y += xStat.Size.Height;
	xMain.Size.Height = xHelp.Bounds.Top - xMain.Location.Y;
	
	xStat.Show();
	GoToMainMenu(false);
	xMain.Show();
	xHelp.Show();
}

//======================================================================
// [FUNC] Initiates a new game.
private static void StartGame()
{
	if (!IsGameInProgress) {
		Random rnd = new Random();
		ConsoleKey[] keys = new ConsoleKey[] {ConsoleKey.LeftArrow,
			ConsoleKey.UpArrow, ConsoleKey.RightArrow, ConsoleKey.DownArrow};
		
		xMain.ClearText();
		xMain.Text = "LOADING NEW GAME...";
		xMain.CenterText();
		xMain.DrawText();
		
		xHelp.ClearText();
		
		// Shuffle the "tokens"
		for (int i = 0; i < 0xff; i++)
			MoveToken(keys[rnd.Next(4)]);
		
		for (int i = 0; i < Token.Length; i++)
			Token[i].Show(true);
		
		IsGameInProgress = true;
		ResumeGame();
	}
}

//======================================================================
// [FUNC] Resumes gameplay or quits back to main menu. Yeah, it's a
// horrible name for what it does. But "Anything Goes" -- Willie Scott.
private static void ResumeGame()
{
	if (IsGameInProgress) {
		xMain.ClearText();
		xMain.Text = "+ GAME IN PROGRESS +";
		xMain.CenterText();
		xMain.DrawText();
		
		xHelp.ClearText();
		xHelp.Text = "F1=HELP         E=CHECK          ESC=QUIT";
		xHelp.DrawText();
	} else {
		GoToMainMenu(false);
	}
}

//======================================================================
// [FUNC] Quits from the current screen to the main menu. A lil mistake
// in the program flow and I'd have awarded myself a stack overflow medal.
private static void GoToMainMenu(bool isRestart)
{
	if (IsGameInProgress || isRestart) {
		xMain.ClearText();
		xMain.Text = isRestart ?
			"RESTARTING GAME..." : "QUITTING TO MAIN MENU...";
		xMain.CenterText();
		xMain.DrawText();
		xHelp.ClearText();
		
		PlayerMoves = 0;
		UpdateMoves();
		
		for (int i = 0; i < Token.Length; i++) {
			for (int j = 0; j < 5; j++) {
				Console.SetCursorPosition(Token[i].Location.X + 2,
					Token[i].Location.Y + j + 1);
				Console.Write("".PadRight(6, TokenMagicChar));
				Thread.Sleep(10);
			}
		}
		
		IsGameInProgress = false;
		if (isRestart)
			return;
	}
	
	xMain.ClearText();
	xMain.Text = "+  MAIN GAME MENU  +";
	xMain.CenterText();
	xHelp.Text = "F5=START      F2=HIGH SCORES     ESC=QUIT";
	
	xMain.DrawText();
	xHelp.DrawText();
}

//======================================================================
// [FUNC] Displays a help message on how to play the game.
private static void ShowHelp()
{
	xMain.ClearText();
	xMain.TextEx = new String[] {
		"  USE THE ARROW KEYS TO MOVE '0' AROUND  ",
		"",
		"    TILL ALL THE NUMBERS ARE ARRANGED    ",
		"",
		" SEQUENTIALLY FROM '1' TO '0' THEN PRESS ",
		"",
		"   KEY 'E' TO CHECK IF YOU'RE CORRECT.   ",
		"",
		"",
		"",
		"     THE SWAG: LESS NUMBER OF MOVES!     "
	};
	xMain.Padding = new Point(1, 1);
	xMain.DrawText();
	
	xHelp.ClearText();
	xHelp.Text = "ESC=RESUME GAME";
	xHelp.DrawText();
	
	// Let me trick the untrained eye ;)
	while (Console.ReadKey(true).Key != ConsoleKey.Escape);
		ResumeGame();
}

//======================================================================
// [FUNC] Displays the game's high scores.
private static void ShowHighScores()
{
	string[] arr = new string[HighScores.Length - 1];
	
	for (int i = 0; i < arr.Length; i += 2) {
		arr[i] = (i / 2 + 1).ToString() + ". " +
			(HighScores[i] + " ").PadRight(MaxPlayerNameLength + 8, '.') +
			" " + HighScores[i+1].PadLeft(4, '0');
		if (i < arr.Length - 2)
			arr[i+1] = "";
	}
	
	xMain.ClearText();
	xMain.TextEx = arr;
	xMain.CenterText();
	xMain.DrawText();
	
	xHelp.ClearText();
	xHelp.Text = "ESC=BACK TO MAIN MENU";
	xHelp.DrawText();
	
	while (Console.ReadKey(true).Key != ConsoleKey.Escape);
		ResumeGame();
}

//======================================================================
// [FUNC] Quits a game in progress and/or exits the application.
private static void QuitGame()
{
	xMain.ClearText();
	xMain.TextEx = new string[] {
		"ARE YOU SURE YOU WANT TO QUIT?",
		"",
		"         ( YES / NO )"
	};
	xMain.CenterText();
	xMain.DrawText();
	
	xHelp.Text = "ESC/N=NO                      ENTER/Y=YES";
	xHelp.DrawText();
	
	do {
		ConsoleKeyInfo keyInfo = Console.ReadKey(true);
		if (keyInfo.Key == ConsoleKey.Enter || keyInfo.Key == ConsoleKey.Y) {
			if (IsGameInProgress)
				GoToMainMenu(false);
			else
				ImStillAlive = false;
			break;
		} else if (keyInfo.Key == ConsoleKey.Escape ||
			keyInfo.Key == ConsoleKey.N) {
			ResumeGame();
			break;
		}
	} while (true);
}

//======================================================================
// [FUNC] Handles the arrow keystrokes and moves the "Token"s on-screen.
private static void MoveToken(ConsoleKey key)
{
	Token swapToken;
	Point location;
	
	switch (key) {
		case ConsoleKey.LeftArrow:
			if (Token[8].CellID % 3 == 1)
				goto end;
			swapToken = GetTokenByCellId(Token[8].CellID - 1);
			break;
		case ConsoleKey.UpArrow:
			if (Token[8].CellID <= 3)
				goto end;
			swapToken = GetTokenByCellId(Token[8].CellID - 3);
			break;
		case ConsoleKey.RightArrow:
			if (Token[8].CellID % 3 == 0)
				goto end;
			swapToken = GetTokenByCellId(Token[8].CellID + 1);
			break;
		case ConsoleKey.DownArrow:
			if (Token[8].CellID >= 7)
				goto end;
			swapToken = GetTokenByCellId(Token[8].CellID + 3);
			break;
		default:
			// In case you don't know I handle only ARROW KEYS!!!
			// Oh.. no need to scream, (s)he can't even hear me :(
			goto end;
	}
	
	Token[8].CellID  ^= swapToken.CellID;
	swapToken.CellID ^= Token[8].CellID;
	Token[8].CellID  ^= swapToken.CellID;
	
	location = Token[8].Location;
	Token[8].Location = swapToken.Location;
	swapToken.Location = location;
	
	if (IsGameInProgress) {
		swapToken.Show(false);
		Token[8].Show(true);
		PlayerMoves++;
		UpdateMoves();
	}
	
	end:
	return;
}

//======================================================================
// [FUNC] Many of the stuff here a hard coded to just cut to the chase
// which may make this code a little too hard (and frustrating?) to
// maintain after some time. But who would ever need this, anyway?
private static void CheckGame()
{
	string[] msg = new string[0];
	
	for (int i = 0; i < Token.Length; i++)
		if (Token[i].CellID != Token[i].Value)
			goto negative;
	
	// Congrats! You just finished the easiest task in the history of creation.
	// And how long did it take you? Well, let's see if you made a high score.
	for (int i = 0, s; i < HighScores.Length; i += 2) {
		if (!Int32.TryParse(HighScores[i+1], out s))
			s = 9999;
		
		if (PlayerMoves <= s) {
			// Push the high score members down the rank.
			for (int j = HighScores.Length - 2; j > i; j -= 2) {
				HighScores[j] = HighScores[j-2];
				HighScores[j+1] = HighScores[j-1];
			}
			HighScores[i] = PlayerName;
			HighScores[i+1] = PlayerMoves.ToString();
			SaveHighScores();
			
			if (i == 0) {
				msg = new string[7];
				msg[0] = "       BON! VOUS ETES UN CHAMPION!       ";
			} else {
				msg = new string[9];
				msg[0] = "     BRAVO!! YOU JUST BECAME THE " +
					Lib.Affixate(i / 2 + 1) + "     ";
				msg[2] = "        BEST PLAYER OF THIS GAME!        ";
			}
			break;
		}
	}
	
	if (msg.Length == 0) {
		msg = new string[7];
		msg[0] = "      KUDOS, YOU FINISHED THE GAME!      ";
	}
	
	for (int i = 1; i < msg.Length; i += 2)
		msg[i] = "";	// Fill in the Null members
	msg[msg.Length-5] = "                   ***";
	msg[msg.Length-3] = "      WOULD YOU LIKE TO PLAY AGAIN?      ";
	msg[msg.Length-1] = "               ( YES / NO )";
	
	ChangeTokensColor(ConsoleColor.Yellow);
	
	xMain.ClearText();
	xMain.TextEx = msg;
	xMain.CenterText();
	xMain.DrawText();
	xHelp.Text = "ENTER/Y=YES                          N=NO";
	xHelp.DrawText();
	
	do switch (Console.ReadKey(true).Key) {
		case ConsoleKey.Y:
		case ConsoleKey.Enter:
			GoToMainMenu(true);
			StartGame();
			return;
		case ConsoleKey.N:
			GoToMainMenu(false);
			return;
	} while (true);
	
	// Why, don't you have eyes? Well, I'll get you some goggles.
	negative:
	ChangeTokensColor(ConsoleColor.Red);
	xMain.ClearText();
	xMain.Text = "OOPS! YOU'VE NOT FINISHED YET!";
	xMain.CenterText();
	xMain.DrawText();
	
	xHelp.ClearText();
	xHelp.Text = "ESC=RESUME GAME                  ENTER=OK";
	xHelp.DrawText();
	
	do switch (Console.ReadKey(true).Key) {
		case ConsoleKey.Escape:
		case ConsoleKey.Enter:
			ChangeTokensColor(Token[0].ForeColor);
			ResumeGame();
			return;
	} while (true);
}

//======================================================================
// [FUNC] Just updates the player's number of moves made on-screen.
private static void UpdateMoves()
{
	Console.ForegroundColor = xStat.ForeColor;
	Console.SetCursorPosition(xStat.Bounds.Right - 6, xStat.Location.Y + 1);
	Console.Write(PlayerMoves.ToString("D4"));
}

//======================================================================
// [FUNC] Returns a reference to the token occupying the given cell ID.
private static Token GetTokenByCellId(int cellID)
{
	for (int i = 0; i < Token.Length; i++)
		if (Token[i].CellID == cellID)
			return Token[i];
	return null;
}

//======================================================================
// [FUNC] Not overly essential but I needed it somewhere along the line.
// It temporarily changes the color of the "token"s to the specified one.
private static void ChangeTokensColor(ConsoleColor newColor)
{
	ConsoleColor oldColor;
	for (int i = 0; i < Token.Length; i++) {
		oldColor = Token[i].ForeColor;
		Token[i].ForeColor = newColor;
		Token[i].Show(false);
		Token[i].ForeColor = oldColor;
	}
}
}
